; PODPROGRAMY OBSUGI KLAWIATURY PC/AT
; Rafa Baranowski (C) 2004
;
; Etykiety procedur:
; 	Ini_klaw_PS2	- inicjalizacja klawiatury
;	Pobierz_ASCII_PS2	- konwersja kodw klawiatury na kody ASCII
;
; Wymagane zmienne:
;	PS2_ZNACZ		- zmienna bajtowa w pamici SRAM
;
; Wymagane podprogramy dodatkowe:
;	"ps2_buf.inc"	- plik podprogramw dla interfejsu PS/2
;


;****************************************************************
;		PROCEDURA INICJALIZUJCA KLAWIATUR PC
;
Ini_klaw_PS2:
	push	R16

	clr	R16
	sts	PS2_ZNACZ, R16	; zerowanie zmiennej stanu klawiatury

	pop	R16
	ret


;****************************************************************
;	PROCEDURA KONWERTUJCA KODY KLAWIATURY PC NA KODY ASCII
;
; Parametry:
;	PS2_BUF_O	- bufor odbiorczy interfejsu PS/2
;	PS2_WSK_RBO	- wskanik odczytu bufora odbiorczego PS/2
;
; Wartoci wyjciowe:
;	T		- znacznik powodzenia (SREG)
;	R16		- kod ASCII ostatnio nacinitego klawisza (o ile T=1)
;	PS2_ZNACZ	- zmienna znacznikw stanu klawiatury (SRAM)
;
; Korzysta z podprogramw:
;	Pobierz_kod_PS2	- z pliku "ps2_buf.inc"
;
					; etykiety wybranych kodw przesyanych przez klawiatur
.EQU PS2_K_PUSC	= 0xF0	;   zwiastun klawisza puszczonego
.EQU PS2_K_SPEC	= 0xE0	;   zwiastun klawisza specjalnego
.EQU PS2_K_DZIALA	= 0xAA	;   klawiatura uruchomiona ("self test passed")
.EQU PS2_K_ERROR1	= 0x00	;   kod bdu (np. przepenienia bufora)
.EQU PS2_K_ERROR2	= 0xFF	;   kod bdu (np. przepenienia bufora)
.EQU PS2_K_SHIFT1	= 0x12	;   kod lewego klawisza Shift
.EQU PS2_K_SHIFT2	= 0x59	;   kod prawego klawisza Shift
					; etykiety znacznikw w zmiennej stanu PS2_ZNACZ
.EQU PS2_Z_PUSC	= 0		;   znacznik oczekiwania na kod klawisza puszczonego
.EQU PS2_Z_SPEC	= 1		;   znacznik oczekiwania na kod klawisza specjalnego
.EQU PS2_Z_SHIFT	= 2		;   znacznik przytrzymania klawisza Shift

Pobierz_ASCII_PS2:
	push	R17
	push	R18
	push	ZL
	push	ZH

Pob_A_PS2_pocz:
	rcall	Pobierz_kod_PS2	; pobierz bajt z bufora odbiorczego interfejsu PS/2 do R16
	brts	Pob_A_PS2_1		; jeli bufor PS/2 by pusty (T=0), to
	clt				;   zeruj znacznik powodzenia T
	rjmp	Pob_A_PS2_koniec	;   i zakocz procedur

Pob_A_PS2_1:
	cpi	R16, PS2_K_PUSC	; jeli odebrano zwiastun kodu klawisza puszczonego, to..
	brne	Pob_A_PS2_2
	ldi	R17, 1<<PS2_Z_PUSC
	rcall	Pob_A_PS2_ust_zn	;   ustaw znacznik oczekiwania na kod klawisza puszczonego				
	rjmp	Pob_A_PS2_pocz	;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_2:
	cpi	R16, PS2_K_SPEC	; jeli odebrano zwiastun kodu klawisza specjalnego, to 
	brne	Pob_A_PS2_3
	ldi	R17, (1<<PS2_Z_SPEC)
	rcall	Pob_A_PS2_ust_zn	;   ustaw znacznik oczekiwania na kod klawisza specjalnego
	rjmp	Pob_A_PS2_pocz	;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_3:
	cpi	R16, PS2_K_DZIALA	; jeli odebrano kod uruchomienia klawiatury, to
	brne	Pob_A_PS2_4
	rcall	Ini_klaw_PS2	;   zainicjalizuj zmienne klawiatury
	rjmp	Pob_A_PS2_pocz	;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_4:
	cpi	R16, PS2_K_ERROR1	; jeli odebrano kod bdu nr 1, to
	brne	Pob_A_PS2_5
	rcall	Ini_klaw_PS2	;   zainicjalizuj zmienne klawiatury
	rjmp	Pob_A_PS2_pocz	;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_5:
	cpi	R16, PS2_K_ERROR2	; jeli odebrano kod bdu nr 2, to
	brne	Pob_A_PS2_6
	rcall	Ini_klaw_PS2	;   zainicjalizuj zmienne klawiatury
	rjmp	Pob_A_PS2_pocz	;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_6:
	ldi	ZH, high(PS2_kl_znaki)	; zaaduj Z adresem pocztku tablicy konwersji kodw
	ldi	ZL, low(PS2_kl_znaki)	;   klawiszy na kody ASCII
	ldi	R17, 0
	add	ZL, R16
	adc	ZH, R17			; dodaj do Z warto kodu klawisza wcinitego
	lsl	ZL				
	rol	ZH				; przystosuj Z dla instrukcji lpm (przesuwanie w lewo)

	lds	R17, PS2_ZNACZ		; zapisz R17 znacznikami stanu klawiatury
	
	sbrc	R17, PS2_Z_PUSC		; blok wyboru akcji w zalenoci od stanu klawiatury:
	rjmp	Pob_A_PS2_dla_pusc	;   jeli odebrano kod klawisza puszczonego, to skocz
	sbrc	R17, PS2_Z_SPEC
	rjmp	Pob_A_PS2_dla_spec	;   jeli odebrano kod klawisza specjalnego, to skocz
						; 
	sbrc	R17, PS2_Z_SHIFT		;   jeli odebrano kod klawisza przy wcinitym Shift, to
	adiw	ZL, 1				;     zwiksz Z o 1 (z tablicy konwersji odczytywane
						;     bd "due" kody ASCII, bo zajmuj one starsze
						;	bajty tej tablicy
	cpi	R16, PS2_K_SHIFT1		;   jeli odebrano kod lewego klawisza Shift,
	breq	Pob_A_PS2_dla_shift	;     to skocz do obsugi tego zdarzenia
	cpi	R16, PS2_K_SHIFT2		;   jeli odebrano kod prawego klawisza Shift,
	breq	Pob_A_PS2_dla_shift	;     to skocz do obsugi tego zdarzenia

						;   jeli jednak odebrano kod "zwyczajnego" klawisza, to
	lpm	R16, Z			;     przekonwertuj go na kod ASCII i zapisz w R16,
	set					;     ustaw znacznik powodzenia T

Pob_A_PS2_koniec:				;     i zakocz procedur
	pop	ZH
	pop	ZL
	pop	R18
	pop	R17
	ret

Pob_A_PS2_dla_shift:			; przy odbiorze kodu klawisza Shift:
	ldi	R17, (1<<PS2_Z_SHIFT)
	rcall	Pob_A_PS2_ust_zn		;   ustaw znacznik wcinicia klawisza Shift
	rjmp	Pob_A_PS2_pocz		;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_dla_pusc:			; przy odbiorze kodu klawisza puszczonego:
	ldi	R17, (1<<PS2_Z_PUSC)|(1<<PS2_Z_SHIFT)

	cpi	R16, PS2_K_SHIFT1		;   sprawd, czy puszczony zosta lewy Shift
	breq	Pob_A_PS2_dla_pusc_1
	cpi	R16, PS2_K_SHIFT2		;   sprawd, czy puszczony zosta prawy Shift,
	breq	Pob_A_PS2_dla_pusc_1

	cbr	R17, 1<<PS2_Z_SHIFT
Pob_A_PS2_dla_pusc_1:
	rcall	Pob_A_PS2_zer_zn		;   zeruj znacznik oczekiwania na kod klawisza 
						;     puszczonego oraz ewentualnie przytrzymania
						;     kl. Shift (jeli zosta puszczony)
	rjmp	Pob_A_PS2_pocz		;   i sprbuj dalej odczytywa bufor PS/2

Pob_A_PS2_dla_spec:			; przy odbiorze kodu klawisza specjalnego:
	ldi	R17, (1<<PS2_Z_SPEC)
	rcall	Pob_A_PS2_zer_zn		;   zeruj znacznik oczekiwania na kod kl. spec.
	rjmp	Pob_A_PS2_pocz		;   i sprbuj dalej odczytywa bufor PS/2


					; podprocedury dodatkowe:
Pob_A_PS2_ust_zn:			;   podprogram ustawiajcy znaczniki zmiennej PS2_ZNACZ,
	lds	R16, PS2_ZNACZ	;     ktrym odpowiadaj wysokie bity parametru R17
	or	R16, R17		;     (uywa R16!)
	sts	PS2_ZNACZ, R16
	ret

Pob_A_PS2_zer_zn:			;   podprogram zerujcy znaczniki zmiennej PS2_ZNACZ,
	lds	R16, PS2_ZNACZ	;     ktrym odpowiadaj wysokie bity parametru R17
	com	R17			;     (uywa R16!)
	and	R16, R17
	sts	PS2_ZNACZ, R16
	ret


					; tablica konwersji kodw klawiszy na kody ASCII
					;   pod adresami wyznaczonymi przez kody klawiszy znajduj
					;   si odpowiednie kody ASCII; modsza cz kadego
					;   sowa to "may" kod ASCII, starsza - "duy" (dla shift-u)
PS2_kl_znaki:
	.ORG PS2_kl_znaki+0x76 \ .DB 0x1B,0x1B	; ESC
	.ORG PS2_kl_znaki+0x05 \ .DB "??"		; F1
	.ORG PS2_kl_znaki+0x06 \ .DB "??"		; F2
	.ORG PS2_kl_znaki+0x04 \ .DB "??"		; F3
	.ORG PS2_kl_znaki+0x0C \ .DB "??"		; F4
	.ORG PS2_kl_znaki+0x03 \ .DB "??"		; F5
	.ORG PS2_kl_znaki+0x0B \ .DB "??"		; F6
	.ORG PS2_kl_znaki+0x83 \ .DB "??"		; F7
	.ORG PS2_kl_znaki+0x0A \ .DB "??"		; F8
	.ORG PS2_kl_znaki+0x01 \ .DB "??"		; F9
	.ORG PS2_kl_znaki+0x09 \ .DB "??"		; F10
	.ORG PS2_kl_znaki+0x78 \ .DB "??"		; F11
	.ORG PS2_kl_znaki+0x07 \ .DB "??"		; F12
	.ORG PS2_kl_znaki+0x0E \ .DB "`~"		; `
	.ORG PS2_kl_znaki+0x16 \ .DB "1!"		; 1
	.ORG PS2_kl_znaki+0x1E \ .DB "2@"		; 2
	.ORG PS2_kl_znaki+0x26 \ .DB "3",0x23	; 3
	.ORG PS2_kl_znaki+0x25 \ .DB "4$"		; 4
	.ORG PS2_kl_znaki+0x2E \ .DB "5%"		; 5
	.ORG PS2_kl_znaki+0x36 \ .DB "6^"		; 6
	.ORG PS2_kl_znaki+0x3D \ .DB "7&"		; 7
	.ORG PS2_kl_znaki+0x3E \ .DB "8*"		; 8
	.ORG PS2_kl_znaki+0x46 \ .DB "9("		; 9
	.ORG PS2_kl_znaki+0x45 \ .DB "0)"		; 0
	.ORG PS2_kl_znaki+0x4E \ .DB "-_"		; -
	.ORG PS2_kl_znaki+0x55 \ .DB "=+"		; =
	.ORG PS2_kl_znaki+0x5D \ .DB "\|"		; \
	.ORG PS2_kl_znaki+0x66 \ .DB 0x08,0x08	; Backspace
	.ORG PS2_kl_znaki+0x0D \ .DB "??"		; Tab
	.ORG PS2_kl_znaki+0x15 \ .DB "qQ"		; Q
	.ORG PS2_kl_znaki+0x1D \ .DB "wW"		; W
	.ORG PS2_kl_znaki+0x24 \ .DB "eE"		; E
	.ORG PS2_kl_znaki+0x2D \ .DB "rR"		; R
	.ORG PS2_kl_znaki+0x2C \ .DB "tT"		; T
	.ORG PS2_kl_znaki+0x35 \ .DB "yY"		; Y
	.ORG PS2_kl_znaki+0x3C \ .DB "uU"		; U
	.ORG PS2_kl_znaki+0x43 \ .DB "iI"		; I
	.ORG PS2_kl_znaki+0x44 \ .DB "oO"		; O
	.ORG PS2_kl_znaki+0x4D \ .DB "pP"		; P
	.ORG PS2_kl_znaki+0x54 \ .DB "[{"		; [
	.ORG PS2_kl_znaki+0x5B \ .DB "]}"		; ]
	.ORG PS2_kl_znaki+0x58 \ .DB "??"		; Caps Lock
	.ORG PS2_kl_znaki+0x1C \ .DB "aA"		; A
	.ORG PS2_kl_znaki+0x1B \ .DB "sS"		; S
	.ORG PS2_kl_znaki+0x23 \ .DB "dD"		; D
	.ORG PS2_kl_znaki+0x2B \ .DB "fF"		; F
	.ORG PS2_kl_znaki+0x34 \ .DB "gG"		; G
	.ORG PS2_kl_znaki+0x33 \ .DB "hH"		; H
	.ORG PS2_kl_znaki+0x3B \ .DB "jJ"		; J
	.ORG PS2_kl_znaki+0x42 \ .DB "kK"		; K
	.ORG PS2_kl_znaki+0x4B \ .DB "lL"		; L
	.ORG PS2_kl_znaki+0x4C \ .DB 0x3B,":"	; ;
	.ORG PS2_kl_znaki+0x52 \ .DB 0x60,0x22	; '
	.ORG PS2_kl_znaki+0x5A \ .DB 0x0D,0x0A	; Enter	
	.ORG PS2_kl_znaki+0x12 \ .DB "??"		; Shift (lewy)
	.ORG PS2_kl_znaki+0x1A \ .DB "zZ"		; Z
	.ORG PS2_kl_znaki+0x22 \ .DB "xX"		; X
	.ORG PS2_kl_znaki+0x21 \ .DB "cC"		; C
	.ORG PS2_kl_znaki+0x2A \ .DB "vV"		; V
	.ORG PS2_kl_znaki+0x32 \ .DB "bB"		; B
	.ORG PS2_kl_znaki+0x31 \ .DB "nN"		; N
	.ORG PS2_kl_znaki+0x3A \ .DB "mM"		; M
	.ORG PS2_kl_znaki+0x41 \ .DB ",<"		; ,
	.ORG PS2_kl_znaki+0x49 \ .DB ".>"		; .
	.ORG PS2_kl_znaki+0x4A \ .DB "/?"		; /
	.ORG PS2_kl_znaki+0x59 \ .DB "??"		; Shift (prawy)
	.ORG PS2_kl_znaki+0x14 \ .DB "??"		; Ctrl (lewy)
	.ORG PS2_kl_znaki+0x11 \ .DB "??"		; Alt (lewy)
	.ORG PS2_kl_znaki+0x29 \ .DB "  "		; Space
	.ORG PS2_kl_znaki+0x7E \ .DB "??"		; Scroll Lock
	.ORG PS2_kl_znaki+0x77 \ .DB "??"		; Num Lock
	.ORG PS2_kl_znaki+0x7C \ .DB "**"		; * (num)
	.ORG PS2_kl_znaki+0x7B \ .DB "--"		; - (num)
	.ORG PS2_kl_znaki+0x6C \ .DB "77"		; 7 (num)
	.ORG PS2_kl_znaki+0x75 \ .DB "88"		; 8 (num)
	.ORG PS2_kl_znaki+0x7D \ .DB "99"		; 9 (num)
	.ORG PS2_kl_znaki+0x6B \ .DB "44"		; 4 (num)
	.ORG PS2_kl_znaki+0x73 \ .DB "55"		; 5 (num)
	.ORG PS2_kl_znaki+0x74 \ .DB "66"		; 6 (num)
	.ORG PS2_kl_znaki+0x69 \ .DB "11"		; 1 (num)
	.ORG PS2_kl_znaki+0x72 \ .DB "22"		; 2 (num)
	.ORG PS2_kl_znaki+0x7A \ .DB "33"		; 3 (num)
	.ORG PS2_kl_znaki+0x70 \ .DB "00"		; 0 (num)
	.ORG PS2_kl_znaki+0x71 \ .DB ",,"		; , (num)
	.ORG PS2_kl_znaki+0x79 \ .DB "++"		; + (num)

	.ORG PS2_kl_znaki+0x84
